# (c) Copyright 2006-2008. CodeWeavers, Inc.
package CXRPM;
use strict;

use CXLog;
use CXUtils;


#####
#
# Static functions
#
#####

sub compute_package_name($)
{
    my ($name)=@_;
    $name =~ s/[ <>=]/_/g;
    return $name;
}

sub escape_file_path($)
{
    my ($path)=@_;
    # This is the status with rpm 4.4.2.3:
    # - Paths containing a space must be enclosed in quotes. Quoting just the
    #   space or any form of escaping does not work.
    # - '*' must be escaped by enclosing it in square brackets. It will still
    #   trigger a 'file listed twice' warning but it will work regardless.
    # - '!', '#', '$', '%', '&', '(', ')', ';', '<', '>', '[', '\', ']', '`',
    # '{', '|' and '}' also trigger the same warning but don't need escaping.
    # - ''' and '"' must not be quoted and must not be escaped.
    # - If a path contains both a space and a single or double quote...
    #   well, let's hope this does not happen.
    $path =~ s/(\*)/[$1]/g;
    return "\"$path\"" if ($path =~ / /);
    return $path;
}


#####
#
# The CXRPM class
#
#####

sub new($)
{
    my ($class)=@_;
    my $self={ };
    bless $self, $class;

    $self->{rpmbuild}=CXUtils::cxwhich($ENV{PATH}, "rpmbuild");
    if (!defined $self->{rpmbuild})
    {
        $self->{warn}="You may need to install some rpm development " .
                      "packages such as rpm-build";
        $self->{rpmbuild}=CXUtils::cxwhich($ENV{PATH}, "rpm");
        if (!defined $self->{rpmbuild})
        {
            cxerr("unable to find rpmbuild or rpm - $self->{warn}\n");
            cxerr("cannot build the RPM packages\n");
        }
    }

    return $self;
}

sub get_rpm_version($)
{
    my ($self)=@_;
    return cxbackquote(shquote_string($self->{rpmbuild}) . " --version");
}

sub create_buildroot($$$)
{
    my ($self, $src, $buildroot)=@_;
    # On SuSE 8.0 rpm 3.0.6 has a bug which is triggered
    # by using hardlinks to copy the source directory
    my $nolinks=($self->get_rpm_version() =~ /\s3\./);

    if (!cxmkpath($buildroot, 0700))
    {
        cxerr("unable to create the '$buildroot' directory: $@\n");
        return 0;
    }
    my $mode=(stat($src))[2] & 07777;
    chmod($mode, $buildroot);

    my @dirs=("");
    while (@dirs)
    {
        my $dir=shift @dirs;
        cxlog("$dir\n");

        my $dh;
        if (!opendir($dh, "$src/$dir"))
        {
            cxerr("unable to open the '$src/$dir' directory: $!\n");
            return 0;
        }
        foreach my $dentry (readdir $dh)
        {
            next if ($dentry =~ /^\.\.?$/);
            $dentry="$dir$dentry";

            if (-l "$src/$dentry")
            {
                my ($dst, $gunzip)=$self->map_file($src, $dentry);
                if (!defined $dst)
                {
                    cxlog("ignoring $dentry\n");
                    next;
                }
                if ($gunzip)
                {
                    cxerr("not decompressing symbolic links '$dentry'\n");
                    return 0;
                }
                my $lnk=readlink "$src/$dentry";
                if (!symlink $lnk, "$buildroot/$dst")
                {
                    cxerr("unable to symlink '$buildroot/$dst' to '$lnk': $!\n");
                    return 0;
                }
            }
            elsif (-d _)
            {
                my $mode=(stat(_))[2] & 07777;
                my $dst=$self->map_directory($src, $dentry);
                next if (!defined $dst);
                if (!mkdir("$buildroot/$dst", 0700))
                {
                    cxerr("unable to create the '$buildroot/$dst' directory: $!\n");
                    return 0;
                }
                chmod($mode, "$buildroot/$dst");
                push @dirs, "$dentry/";
            }
            else
            {
                my ($dst, $gunzip)=$self->map_file($src, $dentry);
                next if (!defined $dst);
                require File::Copy;
                if (($nolinks or $gunzip or !link "$src/$dentry", "$buildroot/$dst") and
                    !File::Copy::copy("$src/$dentry", "$buildroot/$dst"))
                {
                    cxerr("unable to copy '$src/$dentry' to '$buildroot/$dst': $!\n");
                    return 0;
                }
                if ($gunzip and cxsystem("gunzip", "$buildroot/$dst"))
                {
                    cxerr("unable to decompress '$buildroot/$dst'\n");
                    return 0;
                }
            }
        }
        closedir($dh);
    }
    return 1;
}

sub build($$;$)
{
    my ($self, $specfile, $arch)=@_;
    return 1 if (!defined $self->{rpmbuild});

    $arch=($arch ? "--target $arch" : "");

    my $cmd=shquote_string($self->{rpmbuild}) .
            " -bb --define " . shquote_string("_rpmdir " . cxdirname($specfile)) .
            " $arch " . shquote_string($specfile) . " >/dev/null";
    my $rc=cxsystem($cmd);
    if ($rc and $self->{warn})
    {
        cxwarn("$self->{rpmbuild} failed - $self->{warn}\n");
    }
    return ($rc ? 1 : 0);
}

return 1;
